/* copyright 2002 Oliver Dain
   This file is part of Resistor.

    Resistor is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Resistor; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ResistorWin.h"
#include <qcombobox.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qregexp.h>
#include <qpushbutton.h>
const int pix_size = 15;

ResistorWin::ResistorWin(void)
{
	QPixmap pix(pix_size, pix_size);
	FillBand(Band1);
	FillBand(Band2);
	FillBand(Band3);
	//Band 4 is the tolerance band so it has only 2 values

	pix.fill(QColor(184,179,38));
	Band4->insertItem(pix, "Gold");
	cbTolerance->insertItem(pix, "5%");
	pix.fill(gray);
	Band4->insertItem(pix, "Silver");
	cbTolerance->insertItem(pix, "10%");
}

ResistorWin::~ResistorWin(void)
{
}

void ResistorWin::FillBand(QComboBox *box)
{
	QPixmap pix(pix_size, pix_size);
	pix.fill(black);
	box->insertItem(pix, "Black");
	pix.fill(QColor(113, 98, 57));
	box->insertItem(pix, "Brown");
	pix.fill(red);
	box->insertItem(pix, "Red");
	pix.fill(QColor(234,155,19));
	box->insertItem(pix, "Orange");
	pix.fill(yellow);
	box->insertItem(pix, "Yellow");
	pix.fill(green);
	box->insertItem(pix, "Green");
	pix.fill(blue);
	box->insertItem(pix, "Blue");
	pix.fill(magenta);
	box->insertItem(pix, "Violet");
	pix.fill(gray);
	box->insertItem(pix, "Gray");
	pix.fill(white);
	box->insertItem(pix, "White");
}

void ResistorWin::btnBandToNum_clicked()
{
	QString suffix;
	QString prefix;
	QString final;
	//number of places to shift decimal
	int num_zeros = 0;
	//first figure out the suffix
	if (Band3->currentItem() < 2)
	{
		suffix = QString::null;
		num_zeros = Band3->currentItem();
	}
	else if (Band3->currentItem() < 5)
	{
		suffix = "k";
		num_zeros = Band3->currentItem() - 3;
	}
	else
	{
		suffix = "M";
		num_zeros = Band3->currentItem() - 6;
	}
	
	if (num_zeros < -1)
	{
		//I did something wrong
		QMessageBox::warning(this, "AAAH!", "I did something wrong!");
		exit(1);
	}
	if (num_zeros == -1)
	{
		prefix = QString("%1.%2").arg(Band1->currentItem()).arg(Band2->currentItem());
	}
	else 
	{
		prefix = QString("%1%2").arg(Band1->currentItem()).arg(Band2->currentItem());
		for (int i=0; i<num_zeros; i++)
		{
			prefix = prefix + "0";
		}
	}
	leValue->setText(QString("%1 %2").arg(prefix).arg(suffix));
	cbTolerance->setCurrentItem(Band4->currentItem());
}

void ResistorWin::btnNumToBand_clicked()
{
	//figure out the color bands
	QString val = leValue->text();
	QString tmp;
	int b1=0, b2=0;
	int exp = 0;
	int pos;
	bool flag_valid = false;
	//make sure its a valid resistor value
	QRegExp rx;
	//There are 3 valid syntaxes -- check 'em all
	rx.setPattern("^ *0*([1-9]{1,2})(0*)\\.?0* *([kKmM]?) *$");
	pos = rx.search(val);
	if (pos != -1)
	{
		flag_valid = true;
		tmp = rx.cap(1).at(0);
		b1 = tmp.toInt();
		if (rx.cap(1).length() < 2)
			b2 = 0;
		else
		{
			tmp = rx.cap(1).at(1);
			b2 = tmp.toInt();
			exp += 1;
		}
		exp += rx.cap(2).length();
		if (rx.cap(3).length() > 0)
		{
			if ((rx.cap(3) == "k") || (rx.cap(3) == "K"))
				exp += 2;
			else
				exp += 5;
		}
		else
			exp -= 1;
	}
	else
	{
		rx.setPattern("^ *0*([1-9])\\.([1-9]?)0* *([kKMm]?) *$");
		pos = rx.search(val);
		if (pos != -1)
		{
			flag_valid = true;
			tmp = rx.cap(1);
			b1 = tmp.toInt();
			tmp = rx.cap(2);
			if (tmp.length() > 0)
			{
				b2 = tmp.toInt();
				//If we've got a non-zero value after the decimal point
				//and no k or M, then this can't be a valid value
				if (rx.cap(3).length() == 0)
					flag_valid = false;
			}
			else
				b2 = 0;
			tmp = rx.cap(3);
			if (tmp.length() > 0)
			{
				if ((rx.cap(3) == "k") || (rx.cap(3) == "K"))
					exp += 2;
				else
					exp += 5;
			}
		}
		else
		{
			rx.setPattern("^ *0*\\.(0*)([1-9]{1,2})0* *([kKmM]) *$");
			pos = rx.search(val);
			if (pos != -1)
			{
				flag_valid = true;
				exp -= rx.cap(1).length();
				tmp = rx.cap(2).at(0);
				b1 = tmp.toInt();
				tmp = rx.cap(2);
				if (tmp.length() < 2)
					b2 = 0;
				else
				{
					tmp = tmp[1];
					b2 = tmp.toInt();
				}
				tmp = rx.cap(3);
				if ((tmp == "k") || (tmp == "K"))
					exp += 1;
				else
					exp += 4;
			}

		}
	}

	if (flag_valid && (exp >= 0))
	{
		Band1->setCurrentItem(b1);
		Band2->setCurrentItem(b2);
		Band3->setCurrentItem(exp);
		Band4->setCurrentItem(cbTolerance->currentItem());
	}
	else
	{
		QMessageBox::warning(this, "Syntax Error", "That is not a valid resistor value.");
	}

	
}

void ResistorWin::setBand2NumDefault()
{
  btnBandToNum->setDefault(true);
  btnNumToBand->setDefault(false);

}


void ResistorWin::setNum2BandDefault()
{
  btnNumToBand->setDefault(true);
  btnBandToNum->setDefault(false);
}
